package com.yunzhi.gnyy.controller;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import cn.binarywang.wx.miniapp.bean.WxMaUserInfo;
import cn.dev33.satoken.stp.StpUtil;
import com.yunzhi.gnyy.common.*;
import com.yunzhi.gnyy.entity.BdsOrganization;
import com.yunzhi.gnyy.entity.TaPerson;
import com.yunzhi.gnyy.entity.WxUser;
import com.yunzhi.gnyy.service.IBdsOrganizationService;
import com.yunzhi.gnyy.service.ITaPersonService;
import com.yunzhi.gnyy.service.IWxUserService;
import com.yunzhi.gnyy.vo.LoginParam;
import com.yunzhi.gnyy.vo.WxMaAuthParam;
import com.yunzhi.gnyy.vo.WxMaPreload;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

@Api(tags = "小程序登入/登出")
@RestController
@RequestMapping("/wx/{clientId}")
public class WxMaController extends BaseController {

    @Autowired
    WxUtils wxUtils;

    @Autowired
    IBdsOrganizationService iBdsOrganizationService;

    private boolean checkLoginParam(LoginParam loginParam) {
        return !StringUtils.isEmpty(loginParam.getCode())
                && !StringUtils.isEmpty(loginParam.getUserName())
                && !StringUtils.isEmpty(loginParam.getPassword());
    }

    @GetMapping("/preload")
    @ApiOperation(value="小程序登录", notes = "小程序登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean preload(@ApiParam("客户端ID") @PathVariable String clientId,
                                @ApiParam("小程序预加载参数") @RequestParam(required = false) WxMaPreload wxMaPreload) throws Exception {


        return ResponseBean.success(null);
    }

    /**
     * 小程序
     * @param loginParam
     * @return
     * @throws Exception
     */
    @PostMapping("/login")
    @ApiOperation(value="小程序登录", notes = "小程序登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean appLogin(@ApiParam("客户端ID") @PathVariable String clientId,
                                 @ApiParam("登录参数") @RequestBody LoginParam loginParam) throws Exception {

        if (StringUtils.isEmpty(loginParam.getCode())) {
            return ResponseBean.error("CODE 不能为空");
        }

        WxMaService service = wxUtils.getMaService();

        WxMaJscode2SessionResult sessionInfo = service.getUserService().getSessionInfo(loginParam.getCode());
        String openid = sessionInfo.getOpenid();
        String sessionKey = sessionInfo.getSessionKey();

        WxUser wxUser = iWxUserService.getExistBy("openid", openid, false, true);
        if (wxUser == null) {
            wxUser = new WxUser();
            wxUser.setAppId(wxUtils.getAppId());
            wxUser.setOpenid(openid);
            iWxUserService.save(wxUser);
        }

        TaPerson taPerson = null;
        if (!StringUtils.isEmpty(wxUser.getPersonId())) {
            taPerson = iTaPersonService.getExistBy("person_id", wxUser.getPersonId(), false, true);
            if (null != taPerson && !StringUtils.isEmpty(taPerson.getDeptCode())) {
                BdsOrganization org = iBdsOrganizationService.getById(taPerson.getDeptCode());
                taPerson.setDeptName(org.getName());
            }
        }

        StpUtil.login(wxUser.getUserId(), "wx");
        Map<String, Object> res = new HashMap<>();
        res.put("user", wxUser);
        res.put("person", taPerson);
        res.put("token", StpUtil.getTokenValue());
        res.put("sessionKey", sessionKey);

        return ResponseBean.success(res);
    }


    @PutMapping("/auth-user")
    @ApiOperation(value="授权头像", notes = "授权头像", httpMethod = "PUT", response = TaPerson.class)
    public ResponseBean updateUserInfo(@ApiParam("客户端ID") @PathVariable String clientId,
                                       @ApiParam("授权头像的参数") @RequestBody WxMaAuthParam params) throws Exception {
        WxMaService service = wxUtils.getMaService();
        checkAuthParams(service, params);

        // 解密用户信息
        WxMaUserInfo userInfo = service.getUserService().getUserInfo(params.getSessionKey(), params.getEncryptedData(), params.getIv());

        WxUser wxUser = currentWxUser();
        if (null == wxUser) {
            throw new Exception("校验当前人员失败, 请重试");
        }

        wxUser.setAvatar(userInfo.getAvatarUrl());
        wxUser.setNickName(userInfo.getNickName());

        iWxUserService.updateById(wxUser);

        return ResponseBean.success(wxUser);
    }

    @ApiOperation(value="授权手机", notes = "授权手机", httpMethod = "PUT", response = TaPerson.class)
    @PutMapping("/auth-phone")
    public ResponseBean updateUserPhone(@ApiParam("客户端ID") @PathVariable String clientId,
                                        @ApiParam("授权手机的参数") @RequestBody WxMaAuthParam params) throws Exception {

        WxMaService service = wxUtils.getMaService();

        // 解密
        WxMaPhoneNumberInfo phoneNoInfo = service.getUserService().getPhoneNoInfo(params.getSessionKey(), params.getEncryptedData(), params.getIv());

        return ResponseBean.success(phoneNoInfo);
    }


    private void checkAuthParams(WxMaService service, WxMaAuthParam params) throws Exception {
        String signature = params.getSignature();
        String rawData = params.getRawData();
        String encryptedData = params.getEncryptedData();
        String iv = params.getIv();
        String sessionKey = params.getSessionKey();

        if (StringUtils.isEmpty(signature) || StringUtils.isEmpty(rawData) || StringUtils.isEmpty(encryptedData) || StringUtils.isEmpty(iv) || StringUtils.isEmpty(sessionKey)) {
            throw new Exception("缺失授权参数");
        }

        boolean isValid = service.getUserService().checkUserInfo(sessionKey, rawData, signature);
        if (!isValid) {
            throw new Exception("校验授权参数失败, 请重试");
        }
    }
}
